@@ -1,7 +1,7 @@
 /* -*- c-basic-offset: 2 -*- */
 /*
-  Copyright(C) 2017 Brazil
-  Copyright(C) 2019-2020 Sutou Kouhei <kou@clear-code.com>
+  Copyright(C) 2017  Brazil
+  Copyright(C) 2019-2021  Sutou Kouhei <kou@clear-code.com>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -407,7 +407,9 @@ namespace grnarrow {
         grn_id record_id;
         if (array.IsNull(i)) {
           record_id = GRN_ID_NIL;
-          grn_loader_on_no_identifier_error(ctx_, grn_loader_);
+          grn_loader_on_no_identifier_error(ctx_,
+                                            grn_loader_,
+                                            grn_loader_->table);
         } else {
           set_bulk(i);
           record_id = add_record();
@@ -446,11 +448,13 @@ namespace grnarrow {
             requested_record_id = GRN_UINT32_VALUE(&casted_record_id);
           } else {
             auto ctx = ctx_;
+            GRN_DEFINE_NAME(grn_loader_->table);
             grn_obj inspected;
             GRN_TEXT_INIT(&inspected, 0);
             grn_inspect(ctx_, &inspected, &bulk_);
             ERR(GRN_INVALID_ARGUMENT,
-                "<%s>: failed to cast to <UInt32>: <%.*s>",
+                "[table][load][%.*s][%s] failed to cast to <UInt32>: <%.*s>",
+                name_size, name,
                 GRN_COLUMN_NAME_ID,
                 (int)GRN_TEXT_LEN(&inspected),
                 GRN_TEXT_VALUE(&inspected));
@@ -1012,26 +1016,51 @@ namespace grnarrow {
         if (ctx_->rc == GRN_SUCCESS) {
           grn_obj_set_value(ctx_, grn_column_, record_id, &buffer_, GRN_OBJ_SET);
           if (grn_loader_) {
-            grn_loader_on_column_set(ctx_,
-                                     grn_loader_,
-                                     grn_column_,
-                                     record_id,
-                                     nullptr,
-                                     &buffer_);
+            grn_loader_add_record_data data;
+            std::string column_name_buffer;
+            init_grn_loader_add_record_data(&data,
+                                            record_id,
+                                            &buffer_,
+                                            column_name_buffer);
+            grn_loader_on_column_set(ctx_, grn_loader_, &data);
           }
         } else {
           if (grn_loader_) {
-            grn_loader_on_column_set(ctx_,
-                                     grn_loader_,
-                                     grn_column_,
-                                     record_id,
-                                     nullptr,
-                                     visitor.original_value());
+            grn_loader_add_record_data data;
+            std::string column_name_buffer;
+            init_grn_loader_add_record_data(&data,
+                                            record_id,
+                                            visitor.original_value(),
+                                            column_name_buffer);
+            grn_loader_on_column_set(ctx_, grn_loader_, &data);
           }
         }
       }
       return arrow::Status::OK();
     }
+
+    void init_grn_loader_add_record_data(grn_loader_add_record_data *data,
+                                         grn_id record_id,
+                                         grn_obj *value,
+                                         std::string &column_name_buffer)
+    {
+      data->table = grn_loader_->table;
+      data->depth = 0;
+      data->record_value = nullptr;
+      data->id = record_id;
+      data->key = nullptr;
+      auto ctx = ctx_;
+      GRN_DEFINE_NAME(grn_column_);
+      column_name_buffer.assign(name, name_size);
+      auto delimiter_position = column_name_buffer.find(GRN_DB_DELIMITER);
+      if (delimiter_position != std::string::npos) {
+        column_name_buffer.erase(0, delimiter_position + 1);
+      }
+      data->current.column_name = column_name_buffer.data();
+      data->current.column_name_size = column_name_buffer.size();
+      data->current.column = grn_column_;
+      data->current.value = value;
+    }
   };
 
   class FileLoader {
@@ -1659,7 +1688,7 @@ namespace grnarrow {
     }
 
     arrow::Status Close() override {
-      is_open_ = true;
+      is_open_ = false;
       return arrow::Status::OK();
     }
 
@@ -1898,6 +1927,38 @@ namespace grnarrow {
             "failed to add a column value: <" << value << ">");
     }
 
+    void add_column_float32(float value) {
+      auto column_builder =
+        record_batch_builder_->GetFieldAs<arrow::FloatBuilder>(
+          current_column_index_++);
+      auto status = column_builder->Append(value);
+      if (!status.ok()) {
+        return;
+      }
+      std::stringstream context;
+      check(ctx_,
+            status,
+            context <<
+            "[arrow][stream-writer][add-column][float32] " <<
+            "failed to add a column value: <" << value << ">");
+    }
+
+    void add_column_float(double value) {
+      auto column_builder =
+        record_batch_builder_->GetFieldAs<arrow::DoubleBuilder>(
+          current_column_index_++);
+      auto status = column_builder->Append(value);
+      if (!status.ok()) {
+        return;
+      }
+      std::stringstream context;
+      check(ctx_,
+            status,
+            context <<
+            "[arrow][stream-writer][add-column][float] " <<
+            "failed to add a column value: <" << value << ">");
+    }
+
     void add_column_timestamp(grn_timeval value) {
       auto column_builder =
         record_batch_builder_->GetFieldAs<arrow::TimestampBuilder>(
@@ -2654,6 +2715,38 @@ grn_arrow_stream_writer_add_column_uint64(grn_ctx *ctx,
   GRN_API_RETURN(ctx->rc);
 }
 
+grn_rc
+grn_arrow_stream_writer_add_column_float32(grn_ctx *ctx,
+                                           grn_arrow_stream_writer *writer,
+                                           float value)
+{
+  GRN_API_ENTER;
+#ifdef GRN_WITH_APACHE_ARROW
+  writer->writer->add_column_float32(value);
+#else
+  ERR(GRN_FUNCTION_NOT_IMPLEMENTED,
+      "[arrow][stream-writer][add-column][float32] "
+      "Apache Arrow support isn't enabled");
+#endif
+  GRN_API_RETURN(ctx->rc);
+}
+
+grn_rc
+grn_arrow_stream_writer_add_column_float(grn_ctx *ctx,
+                                         grn_arrow_stream_writer *writer,
+                                         double value)
+{
+  GRN_API_ENTER;
+#ifdef GRN_WITH_APACHE_ARROW
+  writer->writer->add_column_float(value);
+#else
+  ERR(GRN_FUNCTION_NOT_IMPLEMENTED,
+      "[arrow][stream-writer][add-column][float] "
+      "Apache Arrow support isn't enabled");
+#endif
+  GRN_API_RETURN(ctx->rc);
+}
+
 grn_rc
 grn_arrow_stream_writer_add_column_timestamp(grn_ctx *ctx,
                                              grn_arrow_stream_writer *writer,
